GSM_INHIBITOR_FLAG_LOGOUT = 1;
GSM_INHIBITOR_FLAG_SWITCH_USER = 2;
GSM_INHIBITOR_FLAG_SUSPEND = 4;
GSM_INHIBITOR_FLAG_IDLE = 8;
GSM_INHIBITOR_FLAG_AUTOMOUNT = 16;
GSM_PRESENCE_STATUS_AVAILABLE = 0;
GSM_PRESENCE_STATUS_INVISIBLE = 1;
GSM_PRESENCE_STATUS_BUSY = 2;
GSM_PRESENCE_STATUS_IDLE = 3;
