#define NEW gsd_updates_manager_new
#define START gsd_updates_manager_start
#define STOP gsd_updates_manager_stop
#define MANAGER GsdUpdatesManager
#include "gsd-updates-manager.h"

#include "test-plugin.h"
